﻿/*Copyright (c) 2017 Grégoire Meyer
 * 
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
 * */

using UnityEngine;
using System.Collections;
using UnityEditor;
using Sigian;

public class PostProcessor : AssetPostprocessor
{

    void OnPostprocessTexture(Texture2D texture)
    {
        string path = assetPath.ToLower();
        bool isInSpritesDirectory = path.IndexOf("/ui/") != -1;
        bool isInNormalDirectory = path.IndexOf("/normalmaps/") != -1;
        bool isAssetRenamed = false;

        string textureName = GetAssetNameFromPath(assetPath);

        if (isInSpritesDirectory)
        {
            TextureImporter textureImporter = (TextureImporter)assetImporter;

            textureImporter.textureType = TextureImporterType.Sprite;
            
            if(textureName != "")
            {
                if(!textureName.ContainsChar('_') || textureName.Split('_')[0] != "UI")
                {
                    isAssetRenamed = true;
                    AssetDatabase.RenameAsset(textureImporter.assetPath, "UI_" + textureName.Split('.')[0]);
                }
            }        
        }

        else if (isInNormalDirectory)
        {
            TextureImporter textureImporter = (TextureImporter)assetImporter;
          
            textureImporter.textureType = TextureImporterType.NormalMap;

            if (textureName != "")
            {
                if (!textureName.ContainsChar('_') || textureName.Split('_')[0] != "NM")
                {
                    isAssetRenamed = true;
                    AssetDatabase.RenameAsset(textureImporter.assetPath, "NM_" + textureName.Split('.')[0]);
                }
            }            
        }

        if (isAssetRenamed)
        {
            texture.Apply();
            AssetDatabase.Refresh();
            Debug.ClearDeveloperConsole();
        }
    }

    void OnPostprocessAudio(AudioClip clip)
    {
        string path = assetPath.ToLower();
        bool isInMusicDirectory = path.IndexOf("/musics/") != -1;
        bool isInSoundsDirectory = path.IndexOf("/sfx/") != -1;
        bool isAssetRenamed = false;

        AudioImporterSampleSettings importerSettings = new AudioImporterSampleSettings();

        string soundName = GetAssetNameFromPath(assetPath);

        if (isInMusicDirectory)
        {
            AudioImporter audioImporter = (AudioImporter)assetImporter;

            importerSettings.loadType = AudioClipLoadType.Streaming;
            importerSettings.compressionFormat = AudioCompressionFormat.Vorbis;
            audioImporter.SetOverrideSampleSettings("Standalone", importerSettings);
            audioImporter.preloadAudioData = false;

            if (soundName != "")
            {
                if (!soundName.ContainsChar('_') || soundName.Split('_')[0] != "MUS")
                {
                    isAssetRenamed = true;
                    AssetDatabase.RenameAsset(audioImporter.assetPath, "MUS_" + soundName.Split('.')[0]);
                }
            }
        }

        else if (isInSoundsDirectory)
        {
            AudioImporter audioImporter = (AudioImporter)assetImporter;

            importerSettings.compressionFormat = AudioCompressionFormat.PCM;
            importerSettings.loadType = AudioClipLoadType.DecompressOnLoad;
            audioImporter.SetOverrideSampleSettings("Standalone", importerSettings);
            audioImporter.preloadAudioData = true;

            if (soundName != "")
            {
                if (!soundName.ContainsChar('_') || soundName.Split('_')[0] != "SFX")
                {
                    isAssetRenamed = true;
                    AssetDatabase.RenameAsset(audioImporter.assetPath, "SFX_" + soundName.Split('.')[0]);
                }
            }
        }
        if (isAssetRenamed)
        {
            AssetDatabase.Refresh();
            Debug.ClearDeveloperConsole();
        }

    }

    string GetAssetNameFromPath(string path)
    {
        string[] choppedPath = path.Split(new char[] { '/', '.' });
        return choppedPath[choppedPath.Length - 2];
    }
}
