﻿/*Copyright (c) 2017 Grégoire Meyer
 * 
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
 * */

using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEditor;
using Sigian;

public class RegroupItems : MonoBehaviour {


    [MenuItem("Sigian/Regroup %G")]
	public static void Regroup()
    {
        GameObject[] selection = Selection.gameObjects;

        if(selection.Length > 0)
        {
            GameObject GOparent = new GameObject("Please rename");
            Undo.RegisterCreatedObjectUndo(GOparent, "Create parent for regroup");

            if (UtilityWindow.isParentCenteringEnabled)
            {
                float totalX = 0;
                float totalY = 0;
                float totalZ = 0;

                for (int j = 0; j < selection.Length; j++)
                {
                    totalX += selection[j].transform.position.x;
                    totalY += selection[j].transform.position.y;
                    totalZ += selection[j].transform.position.z;
                }

                GOparent.transform.position = new Vector3(totalX / selection.Length, totalY / selection.Length, totalZ / selection.Length);
            }

            if (UtilityWindow.isSmartNamingEnabled)
            {
                List<string> choppednames = new List<string>();
                string exception = string.Empty;

                for(int k = 0; k < selection.Length; k++)
                {
                    if (selection[k].name.ContainsChar('(') && selection[k].name.ContainsChar(')'))
                    {
                        choppednames.Add(selection[k].name.Split('(')[0]);
                    }
                    else if (exception != string.Empty)
                    {
                        exception = selection[k].name;
                    }              
                }

                if(choppednames.Count >= (selection.Length - 1) && choppednames.IsConsistent())
                {
                    if (exception != string.Empty)
                        GOparent.name = exception + "s";
                    else
                    {
                        if(choppednames[0].Contains(" "))
                        {
                            GOparent.name = choppednames[0].Split(' ')[0] + "s";

                            if (choppednames[0].ContainsChar('_'))
                            {
                                GOparent.name = GOparent.name.Split('_')[1];
                            }                                    
                        }
                    }
                }
            }

            for (int i = 0; i < selection.Length; i++)
            {
                Undo.SetTransformParent(selection[i].transform, GOparent.transform, "Regroup Selection");
            }

            Selection.activeGameObject = GOparent;
        }
    }
}
