﻿/*Copyright (c) 2018 Grégoire Meyer
 * 
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
 * */

using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEditor;
using Sigian;

[CustomEditor(typeof(SigianTagComponent))]
public class SigianTagCompEditor : Editor {

    private string dummyStringField;

    static SigianTagComponent lastComponent;
    static GameObject GO;

	public override void OnInspectorGUI()
    {
        SigianTagComponent tagComponent = target as SigianTagComponent;

        if(tagComponent.tags.Count > 0)
        {
            EditorGUILayout.LabelField("Tags:", EditorStyles.boldLabel);

            List<string> tempTagsToRemove = new List<string>();

            foreach(string str in tagComponent.tags)
            {
                EditorGUILayout.BeginHorizontal();
                EditorGUILayout.LabelField(str);

                if (GUILayout.Button("X"))
                {
                    tempTagsToRemove.Add(str);
					dummyStringField = string.Empty;
                }

                EditorGUILayout.EndHorizontal();
            }

            foreach(string str in tempTagsToRemove)
            {
                tagComponent.RemoveSigianTag(str);
            }
        }
        else
        {
            EditorGUILayout.Space();
            EditorGUILayout.LabelField("No tags attached to this GameObject", EditorStyles.boldLabel);
        }

        EditorGUILayout.Space();
        EditorGUILayout.BeginHorizontal();
        dummyStringField = EditorGUILayout.TagField(dummyStringField);

        if(GUILayout.Button("Add Tag"))
        {
			if(!dummyStringField.IsNullOrEmpty())
            {
                tagComponent.AddSigianTag(dummyStringField);
                dummyStringField = "Untagged";
                EditorUtility.SetDirty(target);
            }        
        }
        EditorGUILayout.EndHorizontal();

        Repaint();
    }

    private void OnEnable()
    {
        lastComponent = (SigianTagComponent)target;
        GO = lastComponent.gameObject;
    }

    private void OnDestroy()
    {
        if (Application.isEditor && !Application.isPlaying && !EditorApplication.isPlayingOrWillChangePlaymode)
        {
            if((SigianTagComponent)target == null)
            {
                SigianTagManager.ins.database.Remove(GO);
            }
        }
    }

}
