﻿/*Copyright (c) 2018 Grégoire Meyer
 * 
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
 * */

using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEditor;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;
using UnityEngine.SceneManagement;
using Sigian;

public class StickyNoteData : ScriptableObject {

    [System.Serializable]
    private class tempData
    {
        public List<NoteElement> tempList = new List<NoteElement>();
    }

    private static StickyNoteData Instance = null;

    public static StickyNoteData ins
    {
        get
        {
            if (Instance == null)
            {
                Instance = FindObjectOfType<StickyNoteData>();

                if (Instance == null)
                {
                    Instance = (StickyNoteData)AssetDatabase.LoadAssetAtPath("Assets/ScriptableObjects/[Sigian]/NoteData.asset", typeof(StickyNoteData));

                    if (Instance == null)
                    {
                        Instance = CreateInstance<StickyNoteData>();

                        string path = AssetDatabase.GenerateUniqueAssetPath("Assets/ScriptableObjects/[Sigian]/NoteData.asset");
                        AssetDatabase.CreateAsset(Instance, path);
                        AssetDatabase.SaveAssets();
                        AssetDatabase.Refresh();
                        Debug.Log("Sticky Notes manager created at: " + path);
                    }
                }
            }

            return Instance;
        }
    }

    private List<NoteElement> Elements;

    public List<NoteElement> elements
    {
        get
        {
            if(Elements == null)            
                Elements = new List<NoteElement>();
            
            return Elements;
        }
        set
        {
            Elements = value;
        }
    }

    public Dictionary<string, Color> ConvertToDictionary()
    {
        Dictionary<string, Color> temp = new Dictionary<string, Color>();

        foreach (NoteElement elem in elements)
        {
            temp.Add(elem.name, new Color(elem.R, elem.G, elem.B));
        }

        return temp;

    }

    private void OnEnable()
    {
        LoadData();
    }

    public void SaveData()
    {
        tempData temp = new tempData();
        temp.tempList = elements;

        string sceneName = SceneManager.GetActiveScene().name;

        if (string.IsNullOrEmpty(sceneName))
            return;

        if (!Directory.Exists("SigianSaves"))
            Directory.CreateDirectory("SigianSaves");

        BinaryFormatter formatter = new BinaryFormatter();
        FileStream saveFile = File.Create("SigianSaves/stickynotedata_" + sceneName + ".sigian");
        formatter.Serialize(saveFile, temp);
        saveFile.Close();
    }

    public void LoadData()
    {
        tempData temp = new tempData();

        string sceneName = SceneManager.GetActiveScene().name;

        if (string.IsNullOrEmpty(sceneName))
            return;

        if (!File.Exists("SigianSaves/stickynotedata_" + sceneName + ".sigian"))
            return;

        BinaryFormatter formatter = new BinaryFormatter();
        FileStream saveFile = File.Open("SigianSaves/stickynotedata_" + sceneName + ".sigian", FileMode.Open);
        temp = (tempData)formatter.Deserialize(saveFile);
        saveFile.Close();

        elements = temp.tempList;
        SigianCustomHierarchy.Elems = ConvertToDictionary();
    }
}
