﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEditor;
using Sigian;

public class StickyNoteWindow : EditorWindow {

    public StickyNoteData data;
    private string bufferName;
    private Color bufferColor;

    private void Awake()
    {
        data = StickyNoteData.ins;
    }

    private void OnGUI()
    {
        NoteElement tempToDelete = null;

        EditorGUILayout.LabelField("Sticky Note presets:", EditorStyles.boldLabel);
        EditorGUILayout.Space();

        foreach(NoteElement elem in data.elements)
        {
            EditorGUILayout.Space();
            EditorGUILayout.BeginHorizontal();
            elem.name = EditorGUILayout.TextField(elem.name);
            Color tempColor = new Color(elem.R, elem.G, elem.B);
            tempColor = EditorGUILayout.ColorField(tempColor);
            
            if(GUILayout.Button("X"))
            {
                tempToDelete = elem;
                SigianCustomHierarchy.Elems.Remove(elem.name);
            }
            EditorGUILayout.EndHorizontal();
        }

        if(tempToDelete != null)
        {        
            data.elements.Remove(tempToDelete);
            SigianCustomHierarchy.menu = new GenericMenu();
        }

        EditorGUILayout.Space();

        EditorGUILayout.BeginHorizontal();
        bufferName = EditorGUILayout.TextField(bufferName);
        bufferColor = EditorGUILayout.ColorField(bufferColor);

        if(GUILayout.Button("+"))
        {
            if(!bufferName.IsNullOrEmpty() && bufferColor != Color.black)
            {              
                data.elements.Add(new NoteElement(Rect.zero, bufferName, bufferColor, this));
                SigianCustomHierarchy.Elems.Add(bufferName, bufferColor);
                bufferName = string.Empty;
                bufferColor = Color.black;
                EditorUtility.SetDirty(data);
                data.SaveData();
            }      
        }

        EditorGUILayout.EndHorizontal();
        Repaint();
    }

    [MenuItem("Sigian/Sticky Notes")]
    public static void InitWizard()
    {
        StickyNoteWindow wind = (StickyNoteWindow)EditorWindow.GetWindow(typeof(StickyNoteWindow));
        wind.Show();
    }
}


