﻿/*Copyright (c) 2018 Grégoire Meyer
 * 
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
 * */

using System.Collections;
using System.Collections.Generic;
using System;
using UnityEngine;
using UnityEditor;
using UnityEditorInternal;
using Sigian;

public class SelectAllOfTags : EditorWindow {

    public List<string> finalTagsList = new List<string>();

    private int numberOfTags = 1;
    private int toolbarselection = 0;

    public int tagLimitNumber;

    public static bool UsingSigianTagSystem = false;

    string tag1, tag2, tag3, tag4, tag5, tag6, tag7, tag8, tag9;
    private string[] toolbarModes = new string[] { "Use normal tag system", "Use Sigian tag system" };

    [MenuItem("Sigian/Wizards/Select All of Tags")]
    public static void LaunchWizard()
    {
        SelectAllOfTags wind = (SelectAllOfTags)EditorWindow.GetWindow(typeof(SelectAllOfTags));
        wind.Show();
    }

    //aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaah so sloppy
    private void Awake()
    {
        Debug.unityLogger.logEnabled = false;

        if (UsingSigianTagSystem)
            tagLimitNumber = 2;
        else tagLimitNumber = 8;

        tag1 = "Player";
        finalTagsList.Add(tag1);
        finalTagsList.Add(tag2);
        finalTagsList.Add(tag3);
        finalTagsList.Add(tag4);
        finalTagsList.Add(tag5);
        finalTagsList.Add(tag6);
        finalTagsList.Add(tag7);
        finalTagsList.Add(tag8);
        finalTagsList.Add(tag9);
    }

    private void OnGUI()
    {
        EditorGUILayout.Space();
        toolbarselection = GUILayout.Toolbar(toolbarselection, toolbarModes);

        EditorGUILayout.Space();

        using (var group1 = new EditorGUILayout.FadeGroupScope(Convert.ToSingle(toolbarselection == 0)))
        {
            if (group1.visible)
            {
                if (tagLimitNumber != 8)
                    tagLimitNumber = 8;

                EditorGUILayout.LabelField("Normal tag system. Will get all objects with the tags entered.", EditorStyles.miniBoldLabel);
                EditorGUILayout.Space();

                for (int i = 0; i < numberOfTags; i++)
                {
                    EditorGUILayout.BeginHorizontal();
                    EditorGUILayout.LabelField("Tag " + (i + 1));
                    finalTagsList[i] = EditorGUILayout.TagField(finalTagsList[i]);
                    EditorGUILayout.EndHorizontal();
                }

                EditorGUILayout.BeginHorizontal();
                if (numberOfTags <= tagLimitNumber)
                {
                    if (GUILayout.Button("Add Tag"))
                    {
                        numberOfTags++;
                    }
                }

                if (GUILayout.Button("Remove Tag"))
                {
                    numberOfTags--;
                }
                EditorGUILayout.EndHorizontal();

                EditorGUILayout.Space();

                if (GUILayout.Button("Select all of tags"))
                {
                    CalculateSelection_Normal();
                    Close();
                }
            }
            
        }

        using (var group2 = new EditorGUILayout.FadeGroupScope(Convert.ToSingle(toolbarselection == 1)))
        {
            if (group2.visible)
            {
                if (tagLimitNumber != 2)
                    tagLimitNumber = 2;

                if (numberOfTags > 3)
                    numberOfTags = 3;

                EditorGUILayout.LabelField("Sigian tag system. Will only return objects with ALL the tags entered", EditorStyles.miniBoldLabel);
                EditorGUILayout.Space();

                for (int i = 0; i < numberOfTags; i++)
                {
                    EditorGUILayout.BeginHorizontal();
                    EditorGUILayout.LabelField("Tag " + (i + 1));
                    finalTagsList[i] = EditorGUILayout.TagField(finalTagsList[i]);
                    EditorGUILayout.EndHorizontal();
                }

                EditorGUILayout.BeginHorizontal();
                if (numberOfTags <= tagLimitNumber)
                {
                    if (GUILayout.Button("Add Tag"))
                    {
                        numberOfTags++;
                    }
                }

                if (GUILayout.Button("Remove Tag"))
                {
                    numberOfTags--;
                }
                EditorGUILayout.EndHorizontal();

                EditorGUILayout.Space();

                if (GUILayout.Button("Select objects with these tags"))
                {
                    CalculateSelection_Sigian();
                    Close();
                }
            }  
        }
    
        Repaint();
    }

    private void CalculateSelection_Normal()
    {
        List<GameObject> selec = new List<GameObject>();

        foreach (string str in finalTagsList)
        {
            if (str != null)
            {
                GameObject[] temp = GameObject.FindGameObjectsWithTag(str);

                if(temp.Length > 0)
                {
                    foreach (GameObject GO in temp)
                    {
                        selec.Add(GO);
                    }
                }             
            }
        }

        Selection.objects = selec.ToArray();
    }

    private void CalculateSelection_Sigian()
    {

        switch (numberOfTags)
        {
            case 1:
                Selection.objects = SigianTagManager.ins.FindGameObjectsWithSigianTag(finalTagsList[0]);
                break;

            case 2:
                Selection.objects = SigianTagManager.ins.FindGameObjectsWithSigianTag(finalTagsList[0], finalTagsList[1]);
                break;

            case 3:
                Selection.objects = SigianTagManager.ins.FindGameObjectsWithSigianTag(finalTagsList[0], finalTagsList[1], finalTagsList[2]);
                break;

            default:
                Debug.LogError("Sigian Tag search failed! Try searching with less tags.");
                break;
        }


        //Selection.objects = SigianTagManager.ins.FindGameObjectsWithSigianTag();
    }
}

public class ChangeTagAndLayer : EditorWindow
{
    string selectedTag;
    int selectedLayer;

    [MenuItem("Sigian/Wizards/Change Tag and Layer")]
    public static void LaunchWizard()
    {
        ChangeTagAndLayer wind = (ChangeTagAndLayer)EditorWindow.GetWindow(typeof(ChangeTagAndLayer));
        wind.ShowPopup();
    }

    private void OnGUI()
    {
        EditorGUIUtility.labelWidth *= 0.5f;
        EditorGUILayout.BeginHorizontal();
        EditorGUILayout.LabelField("Select new tag");
        selectedTag = EditorGUILayout.TagField(selectedTag);

        EditorGUILayout.Space();

        if(GUILayout.Button("Change tag for selection"))
        {
            foreach(GameObject GO in Selection.gameObjects)
            {
                GO.tag = selectedTag;
            }
        }
        EditorGUILayout.EndHorizontal();
        EditorGUILayout.Space();

        EditorGUILayout.BeginHorizontal();
        EditorGUILayout.LabelField("Select new Layer");
        selectedLayer = EditorGUILayout.LayerField(selectedLayer);

        EditorGUILayout.Space();

        if (GUILayout.Button("Change layer for selection"))
        {
            foreach(GameObject GO in Selection.gameObjects)
            {
                GO.layer = selectedLayer;
            }
        }

        EditorGUILayout.EndHorizontal();
        GUILayout.FlexibleSpace();

        if (GUILayout.Button("CLOSE"))
        {
            Close();
        }
    }

}

public class Congrats : EditorWindow
{
    public static void InitWizard()
    {
        Congrats wiz = (Congrats)EditorWindow.GetWindow(typeof(Congrats));
        wiz.ShowPopup();
    }

    private void OnGUI()
    {
        EditorGUILayout.LabelField("You worked for just as long as you planned!");
        EditorGUILayout.LabelField("Now go get some coffee.");
    }
}

public class SplineBuilder: EditorWindow
{
    private Spline Spl;
    public Spline spline
    {
        get
        {
            return Spl;
        }
        set
        {
            if(Spl != null)
            {
                Spline oldSpline = Spl;
                oldSpline.loaded = false;
                SaveData(oldSpline);
            }

          Spl = value;
        }
    }

    private SplineData data;

    public GameObject[] items;
    List<GameObject> GOs = new List<GameObject>();

    private int Freq;
    public int frequency
    {
        get
        {
            return Freq;
        }
        set
        {
            Freq = value;

            if (GOs == null)
                GOs = new List<GameObject>();

            if (GOs.Count > 0)
            {
                BuildSpline(spline.transform);              
            }


        }
    }

    private float stepsize;
    private float stepOffset;

    public bool lookForward;
    private bool isSplineDetected = false;

	[MenuItem("Sigian/Wizards/Spline Builder")]
    public static void LaunchWizard()
    {
        SplineBuilder wind = (SplineBuilder)EditorWindow.GetWindow(typeof(SplineBuilder));
        wind.ShowUtility();
    }

    private void OnGUI()
    {
        if (Selection.gameObjects.Length == 1)
        {
            if (Selection.activeGameObject.GetComponent<Spline>() != null)
            {
                spline = Selection.activeGameObject.GetComponent<Spline>();
                isSplineDetected = true;
            }
            else
            {
                isSplineDetected = false;
            }
        }
        else
        {
            isSplineDetected = false;
        }          

        if (!isSplineDetected)
        {
            spline = (Spline)EditorGUILayout.ObjectField("Spline", spline, typeof(Spline), true);
        }

        if (isSplineDetected)
        {
            if (spline.GetComponent<SplineData>() != null && !spline.loaded)
            {
                LoadData(spline);
                spline.loaded = true;
            }
        }       

        lookForward = EditorGUILayout.Toggle("Look Forward", lookForward);
        frequency = EditorGUILayout.IntField("Frequency", frequency);

        //Array
        ScriptableObject target = this;
        SerializedObject so = new SerializedObject(target);
        SerializedProperty itemsProperty = so.FindProperty("items");

        EditorGUILayout.PropertyField(itemsProperty, true);
        so.ApplyModifiedProperties();     

        //Apply&Go
        if (frequency > 0 && !items.IsNullOrEmpty())
        {
            if(GOs.Count <= 0)
            {
                if (GUILayout.Button("Build Spline"))
                {
                    BuildSpline(spline.transform);
                }
            }
            else
            {
                EditorGUILayout.LabelField("You can tweak the space between objects with buttons");
                EditorGUILayout.Space();

                EditorGUILayout.BeginHorizontal();

                if (GUILayout.Button("+1"))
                {
                    stepOffset += 1;
                    BuildSpline(spline.transform);
                }
                if (GUILayout.Button("+0.1"))
                {
                    stepOffset += 0.1f;
                    BuildSpline(spline.transform);
                }
                if (GUILayout.Button("+0.01"))
                {
                    stepOffset += 0.01f;
                    BuildSpline(spline.transform);
                }
                if (GUILayout.Button("+0.001"))
                {
                    stepOffset += 0.001f;
                    BuildSpline(spline.transform);
                }
                if (GUILayout.Button("-0.001"))
                {
                    stepOffset -= 0.001f;
                    BuildSpline(spline.transform);
                }
                if (GUILayout.Button("-0.01"))
                {
                    stepOffset -= 0.01f;
                    BuildSpline(spline.transform);
                }
                if (GUILayout.Button("-0.1"))
                {
                    stepOffset -= 0.1f;
                    BuildSpline(spline.transform);
                }
                if (GUILayout.Button("-1"))
                {
                    stepOffset -= 1;
                    BuildSpline(spline.transform);
                }

                EditorGUILayout.EndHorizontal();

                EditorGUILayout.Space();

                if (GUILayout.Button("Apply Tweaks and Go", GUILayout.Height(35)))
                {
                    BuildSpline(spline.transform);
                    Close();
                }
            }           
        }

        Repaint();
    }  

    private void BuildSpline(Transform parent)
    {
        //double security just in case
        if (frequency == 0 || items.IsNullOrEmpty())
            return;

        if (GOs.Count > 0)
            RemovePreviousDecor();

        stepsize = frequency * items.Length;
        if (spline.Loop || stepsize == 1)
        {
            stepsize = 1f / stepsize;
        }
        else
        {
            stepsize = 1f / (stepsize - 1);
        }
        stepsize += stepOffset;

        for (int i = 0, j = 0; j < frequency; j++)
        {
            for(int k = 0; k < items.Length; k++, i++)
            {
                GameObject item = Instantiate(items[k]) as GameObject;
                Vector3 position = spline.GetPoint(i * stepsize);
                item.transform.localPosition = position;

                if (lookForward)
                    item.transform.LookAt(position + spline.GetDirection(i * stepsize));

                item.transform.parent = parent;
                GOs.Add(item.gameObject);
                
            }
        }
        SaveGOs(spline);
        Undo.RecordObjects(GOs.ToArray(), "Build Spline");
    }

    private void RemovePreviousDecor()
    {
        foreach(GameObject item in GOs)
        {
            DestroyImmediate(item, true);
        }

        GOs.Clear();
    }

    private void OnDestroy()
    {
        if(spline != null)
        {
            spline.loaded = false;
            SaveData(spline);
        }     
    }

    private void LoadData(Spline spline)
    {
        data = spline.gameObject.GetComponent<SplineData>();
        frequency = data._frequency;
        GOs = data._GOs;
        items = data._items;
        lookForward = data._lookForward;
        stepOffset = data._stepOffset;
        stepsize = data._stepsize;
    }

    private void SaveData(Spline spline)
    {
        if (spline.gameObject.GetComponent<SplineData>() == null)
            data = spline.gameObject.AddComponent<SplineData>();

        data._frequency = frequency;
        data._items = items;
        data._lookForward = lookForward;
        data._stepOffset = stepOffset;
        data._stepsize = stepsize;
    }

    private void SaveGOs(Spline spline)
    {
        data._GOs = GOs;
    }
}

public class OffsetCalculator : EditorWindow
{
    public GameObject Object1;
    public GameObject Object2;

    private Vector3 offset;

    private bool offsetFound = false;

    private string vectorStringForCopy = "test";

	[MenuItem("Sigian/Wizards/Offset Calculator")]
    public static void LaunchWizard()
    {
        OffsetCalculator wind = (OffsetCalculator)EditorWindow.GetWindow(typeof(OffsetCalculator));
        wind.ShowUtility();
    }

    private void OnGUI()
    {
        if (!offsetFound)
        {
            Object1 = (GameObject)EditorGUILayout.ObjectField("GameObject One", Object1, typeof(GameObject), true);
            Object2 = (GameObject)EditorGUILayout.ObjectField("GameObject Two", Object2, typeof(GameObject), true);

            if(GUILayout.Button("Calculate Offset"))
            {
                CalculateOffset();
            }
        }
        else
        {
            GUILayout.Label("Here is the offset between " + Object1.name + " and " + Object2.name, EditorStyles.boldLabel);
            EditorGUILayout.Space();
            GUILayout.Label("x = " + offset.x.ToString());
            GUILayout.Label("y = " + offset.y.ToString());
            GUILayout.Label("z = " + offset.z.ToString());

            EditorGUILayout.Space();

            GUILayout.Label("You can copy this to put in code");
            vectorStringForCopy = "(" + offset.x.ToString() + ", " + offset.y.ToString() + ", " + offset.z.ToString() + ")";
            vectorStringForCopy = EditorGUILayout.TextField(vectorStringForCopy);

            EditorGUILayout.BeginHorizontal();
            if (GUILayout.Button("New Calculation"))
            {
                ResetWindow();
            }
            if (GUILayout.Button("Close"))
            {
                Close();
            }
            EditorGUILayout.EndHorizontal();
        }

        Repaint();
    }

    private void CalculateOffset()
    {
        offset = Object1.transform.position - Object2.transform.position;
        offsetFound = true;
    }

    private void ResetWindow()
    {
        offsetFound = false;
    }
}


