﻿using System.Collections;
using System.Collections.Generic;
using System;
using UnityEngine;
using UnityEditor;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;
using UnityEngine.SceneManagement;
using Sigian;

[InitializeOnLoad]
public class SigianCustomHierarchy : MonoBehaviour
{
#if UNITY_EDITOR
    //GameObject save system
    private static CustomHierarchyDataBase data = new CustomHierarchyDataBase();
	private static Dictionary<int, string> stickyNoteDatabase = data.database;
	private static Texture2D background = Texture2D.whiteTexture;

    //Note presets
    public static Dictionary<string, Color> Elems = new Dictionary<string, Color>();

    public static GenericMenu menu = new GenericMenu();

	static SigianCustomHierarchy()
	{
		EditorApplication.hierarchyWindowItemOnGUI += SigianView;
		LoadData();
	}

	static void SigianView(int instanceID, Rect selectionRect)
	{
		GameObject GO = EditorUtility.InstanceIDToObject(instanceID) as GameObject;

		if(!stickyNoteDatabase.ContainsKey(instanceID))
		{
			stickyNoteDatabase.Add(instanceID, string.Empty);
		}

		DisplayStickyNote(selectionRect, stickyNoteDatabase[instanceID], GO);
	}

	private static void DisplayStickyNote(Rect selectionRect, string note, GameObject GO)
	{
		Rect r = new Rect(selectionRect);

        if (!note.IsNullOrEmpty())
        {
            Color temp;
            if(Elems.TryGetValue(note, out temp))
            {
                SetColor(background, temp);
                GUI.DrawTexture(r, background);
            }          
        }     


		//text formatting
		r.x += r.width - (9 * note.Length) - 5;
		GUI.Label(r, note, EditorStyles.miniBoldLabel);
             
        r.x += (9 * note.Length) - 10;
        r.width = 20;

		if(Selection.gameObjects.Length > 0)
		{
			if (Selection.gameObjects.Contains(GO))
            {
                if (GUI.Button(r, "+", EditorStyles.miniLabel))
                {
                    //Selection.activeGameObject = GO;
                    menu = new GenericMenu();
                    AddStringItem(menu, "Clear Notes", string.Empty, note);
                    menu.AddSeparator("");

                    foreach (string str in Elems.Keys)
                    {
                        AddStringItem(menu, str, str, note);						
                    }

                    menu.AddSeparator("");
                    menu.AddItem(new GUIContent("Manage sticky notes..."), false, () => EditorApplication.ExecuteMenuItem("Sigian/Sticky Notes"));

                    menu.ShowAsContext();
                }
            }
        }
        SetColor(background, Color.white, 1);
    }

    #region NoteModificationMethods
	private static void ClearNotes()
	{
		GameObject[] GOs = Selection.gameObjects;

		foreach(GameObject GO in GOs)
		{
			if (stickyNoteDatabase.ContainsKey(GO.GetInstanceID()))
            {
                stickyNoteDatabase[GO.GetInstanceID()] = string.Empty;
            }
        }

		SaveData();
	}

    private static void AddNoteToObject(object note)
    {
        if((string)note == string.Empty)
        {
            ClearNotes();
            return;
        }

		GameObject[] GOs = Selection.gameObjects;
		
		foreach(GameObject GO in GOs)
		{
			if (stickyNoteDatabase.ContainsKey(GO.GetInstanceID()))
            {
                stickyNoteDatabase[GO.GetInstanceID()] = (string)note;
            }
            else
            {
                stickyNoteDatabase.Add(GO.GetInstanceID(), (string)note);
            }
		}
        SaveData();
    }

	#endregion

    private static void AddStringItem(GenericMenu menu, string path, string note, string currentNote)
    {
        menu.AddItem(new GUIContent(path), (note == currentNote), AddNoteToObject, note);
    }

	private static void SetColor(Texture2D texture, Color col, float alpha = 0.2f)
	{
        col.a = alpha;

        for(int y = 0; y < texture.height; y++)
		{
			for(int x = 0; x < texture.width; x++)
			{
				texture.SetPixel(x, y, col);
			}
		}

		texture.Apply();
	}

	private static void SaveData()
	{
        data.ElemDatabase.Clear();

        foreach(var TKvalue in Elems)
        {
            float[] temp = new float[] { TKvalue.Value.r, TKvalue.Value.g , TKvalue.Value.b };
            data.ElemDatabase.Add(TKvalue.Key, temp);
        }

        string sceneName = SceneManager.GetActiveScene().name;

        if (string.IsNullOrEmpty(sceneName))
            return;

		if(!Directory.Exists("SigianSaves"))		
			Directory.CreateDirectory("SigianSaves");
		
		BinaryFormatter formatter = new BinaryFormatter();
		FileStream saveFile = File.Create("SigianSaves/customhierarchy_" + sceneName + ".sigian");
		formatter.Serialize(saveFile, data);
		saveFile.Close(); 

		stickyNoteDatabase = data.database;
	}

	private static void LoadData()
	{
        string sceneName = SceneManager.GetActiveScene().name;

        if (string.IsNullOrEmpty(sceneName))
            return;

        if (!File.Exists("SigianSaves/customhierarchy_" + sceneName + ".sigian"))
			return;

		BinaryFormatter formatter = new BinaryFormatter();
		FileStream saveFile = File.Open("SigianSaves/customhierarchy_" + sceneName + ".sigian", FileMode.Open);
		data = (CustomHierarchyDataBase)formatter.Deserialize(saveFile);
		saveFile.Close();

		stickyNoteDatabase = data.database;
	}
#endif
}

[System.Serializable]
public class CustomHierarchyDataBase
{
	public Dictionary<int, string> database = new Dictionary<int, string>();
    public Dictionary<string, float[]> ElemDatabase = new Dictionary<string, float[]>();
}
