﻿/*Copyright (c) 2018 Grégoire Meyer
 * 
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
 * */

using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEditor;
using System.IO;
using Sigian;

[ExecuteInEditMode]
public class SigianTagManager : ScriptableObject {

	private static SigianTagManager Instance = null;

	public static SigianTagManager ins
	{
		get
		{
			if(Instance == null)
			{
				Instance = FindObjectOfType<SigianTagManager>();
               
                if(Instance == null)
                {
                    Instance = (SigianTagManager)AssetDatabase.LoadAssetAtPath("Assets/ScriptableObjects/[Sigian]/TagManager.asset", typeof(SigianTagManager));

                    if (Instance == null)
                    {
                        Instance = CreateInstance<SigianTagManager>();

                        string path = AssetDatabase.GenerateUniqueAssetPath("Assets/ScriptableObjects/[Sigian]/TagManager.asset");
                        AssetDatabase.CreateAsset(Instance, path);
                        AssetDatabase.SaveAssets();
                        AssetDatabase.Refresh();
                        Debug.Log("Sigian tag manager created at: " + path);
                    }
                }				
			}
   
			return Instance;
		}
	}

    public Dictionary<GameObject, string[]> database = new Dictionary<GameObject, string[]>();

    public const int maxTagNumber = 3;

    private string[] tags;

    private void OnEnable()
    {
        tags = UnityEditorInternal.InternalEditorUtility.tags;
    }

    public GameObject FindGameObjectWithSigianTag(string tag)
    {
        if (!tags.Contains(tag))
        {
            Debug.LogWarning(tag + " is not a valid tag! Please add it first!");
            return null;
        }

        foreach(var pair in database)
        {
            if (pair.Value.Contains(tag))
            {
                return pair.Key;
            }
        }
        Debug.LogWarning("error");
        return null;
    }

    public GameObject FindGameObjectWithSigianTag(string tag, string tag2)
    {
        if (!tags.Contains(tag) && !tags.Contains(tag2))
        {
            Debug.LogWarning(tag + " is not a valid tag! Please add it first!");
            return null;
        }

        foreach (var pair in database)
        {
            if(pair.Value.Contains(tag) && pair.Value.Contains(tag2))
            {
                return pair.Key;
            }         
        }
        Debug.LogWarning("error");
        return null;
    }

    public GameObject FindGameObjectWithSigianTag(string tag, string tag2, string tag3)
    {
        if (!tags.Contains(tag) && !tags.Contains(tag2) && !tags.Contains(tag3))
        {
            Debug.LogWarning(tag + " is not a valid tag! Please add it first!");
            return null;
        }

        foreach (var pair in database)
        {
            if (pair.Value.Contains(tag) && pair.Value.Contains(tag2) && pair.Value.Contains(tag3))
            {
                return pair.Key;
            }
        }
        Debug.LogWarning("error");
        return null;
    }

    public GameObject[] FindGameObjectsWithSigianTag(string tag)
    {
        if (!tags.Contains(tag))
        {
            Debug.LogWarning(tag + " is not a valid tag! Please add it first!");
            return null;
        }

        List<GameObject> tempGOList = new List<GameObject>();

        foreach(var pair in database)
        {
            if (pair.Value.Contains(tag))
            {
                tempGOList.Add(pair.Key);
            }
        }

        if(tempGOList.Count == 0)
        {
            return null;
        }
        else
        {
            return tempGOList.ToArray();
        }
    }

    public GameObject[] FindGameObjectsWithSigianTag(string tag, string tag2)
    {
        if(!tags.Contains(tag) && !tags.Contains(tag2))
        {
            Debug.LogWarning(tag + " is not a valid tag! Please add it first!");
            return null;
        }

        List<GameObject> tempGOList = new List<GameObject>();

        foreach (var pair in database)
        {           
            if (pair.Value.Contains(tag) && pair.Value.Contains(tag2))
            {
                tempGOList.Add(pair.Key);
            }
                  
        }

        if (tempGOList.Count == 0)
        {
            return null;
        }
        else
        {
            return tempGOList.ToArray();
        }
    }

    public GameObject[] FindGameObjectsWithSigianTag(string tag, string tag2, string tag3)
    {
        if (!tags.Contains(tag) && !tags.Contains(tag2))
        {
            Debug.LogWarning(tag + " is not a valid tag! Please add it first!");
            return null;
        }

        List<GameObject> tempGOList = new List<GameObject>();

        foreach (var pair in database)
        {
            if (pair.Value.Contains(tag) && pair.Value.Contains(tag2) && pair.Value.Contains(tag3))
            {
                tempGOList.Add(pair.Key);
            }
            
        }

        if (tempGOList.Count == 0)
        {
            return null;
        }
        else
        {
            return tempGOList.ToArray();
        }
    } 

    public void AddTagToDatabase(string tag)
    {
        UnityEditorInternal.InternalEditorUtility.AddTag(tag);
    }

    public void RemoveTagFromDatabase(string tag)
    {
        if (tags.Contains(tag))
        {
            UnityEditorInternal.InternalEditorUtility.RemoveTag(tag);
        }
    }
}
